@@ -60,7 +60,7 @@ impl ConfigurationValue
 			&ConfigurationValue::False => write!(f,"false")?,
 			&ConfigurationValue::Where(ref cv, ref _expr) => write!(f,"{} where FIXME",cv)?,
 			&ConfigurationValue::Expression(ref e) => write!(f,"= {}",e)?,
-			&ConfigurationValue::None => write!(f,"NONE VALUE")?,
+			&ConfigurationValue::None => write!(f,"None")?,
 		};
 		Ok(())
 	}
@@ -95,6 +95,7 @@ impl Display for Expr
 			&Expr::Ident(ref s) => write!(f,"{}",s),
 			&Expr::Member(ref expr,ref s) => write!(f,"{}.{}",expr,s),
 			&Expr::Name(ref expr) => write!(f,"@{}",expr),
+			&Expr::FunctionCall(ref s, ref args) => write!(f,"{}{{{}}}",s,args.iter().map(|(arg_name,arg_expr)|format!("{}:{}",arg_name,arg_expr)).collect::<Vec<String>>().join(",")),
 			_ => write!(f,"fix this expr <{:?}>",self),
 		}
 	}
