@@ -10,13 +10,27 @@ This crate is `caminos-lib`. To use it add `caminos-lib` to your dependencies in
 
 ```toml
 [dependencies]
-caminos-lib = "0.3"
+caminos-lib = "0.4"
 ```
 
 Alternatively, consider whether the binary crate `caminos` fits your intended use.
 
 # Breaking changes
 
+<details>
+
+## [0.3.0] to [0.4.0]
+
+* Added `path` argument to `config::{evaluate,reevaluate}`.
+* File `create_output` and similar now receive in its `results` argument also the experiment indices.
+* routings now return `RoutingNextCandidates`. In addition to the vector of candidates it contains an `idempotent` field to allow some checks and optimizations.
+* Added requirement `VirtualChannelPolicy: Debug`.
+* The `file_main` function now receives a `free_args` parameter. Free arguments of the form `path=value` are used to override the configuration.
+
+## [0.2.0] to [0.3.0]
+
+* Added parameter `cycle` to `Traffic::should_generate`.
+
 ## [0.1.0] to [0.2.0]
 
 * Added methods to `Routing` and `Router` traits to gather statistics.
@@ -26,6 +40,7 @@ Alternatively, consider whether the binary crate `caminos` fits your intended us
 * Now `neighbour_router_iter` must always be used instead of `0..degree()` to check ports to other routers. Note that `degree`  does not give valid ranges when having non-connected ports, as in the case of some irregular topologies as the mesh.
 * `Plugs` now include a `stages` attribute.
 * Removed from the `Topology` interfaz the never used methods `num_arcs`, `average_distance`, `distance_distribution`.
+</details>
 
 # Public Interface
 
@@ -51,6 +66,8 @@ The `directory_main` receives also an `Action`. In the crate `caminos` this is d
 * `remote_check` performs a `check` action in the remote host.
 * `push` compares the local main.cfg with the host remote.cfg. It reports discrepancies and create the remote path if missing.
 * `slurm_cancel` executes a `scancel` with the job ids found in the journal file.
+* `shell` creates the experiment folder with default configuration files. Alternatively, when receiving `--source=another_experiment` it copies the configuration of the other experiment into this one.
+* `pack` forces the creation of a binary.results file and erases the verbose raw results files. In some extreme cases it can reduce a decent amount of space and sped up computations.
 
 
 # Configuration Syntax
@@ -232,7 +249,7 @@ An example of output decription `main.od` is
 ]
 ```
 
-Fot the `tikz` backend to work it is necessary to have a working `LaTeX` installation that included the `pgfplots` package.
+Fot the `tikz` backend to work it is necessary to have a working `LaTeX` installation that included the `pgfplots` package. It is part of the `texlive-pictures` package in some linux distributions.
 
 # Plugging
 
@@ -250,7 +267,7 @@ pub mod router;
 pub mod routing;
 pub mod event;
 pub mod matrix;
-mod output;
+pub mod output;
 pub mod quantify;
 pub mod policies;
 pub mod experiments;
@@ -260,7 +277,7 @@ use std::rc::Rc;
 use std::boxed::Box;
 use std::cell::{RefCell};
 use std::env;
-use std::fs::{File};
+use std::fs::{self,File};
 use std::io::prelude::*;
 use std::io::{stdout};
 use std::collections::{VecDeque,BTreeMap};
@@ -268,11 +285,13 @@ use std::ops::DerefMut;
 use std::path::{Path};
 use std::mem::{size_of};
 use std::fmt::Debug;
+use std::convert::TryFrom;
+use std::cmp::Ordering;
 //use std::default::default;
 //use std::borrow::Cow;
 use rand::{StdRng,SeedableRng};
 
-use config_parser::{ConfigurationValue};
+use config_parser::{ConfigurationValue,Expr};
 use topology::{Topology,new_topology,TopologyBuilderArgument,Location,
 	multistage::{Stage,StageBuilderArgument}};
 use traffic::{Traffic,new_traffic,TrafficBuilderArgument,TrafficError};
@@ -498,6 +517,16 @@ pub struct Phit
 	pub virtual_channel: RefCell<Option<usize>>,
 }
 
+
+#[derive(Quantifiable)]
+#[derive(Debug,Default)]
+pub struct PacketExtraInfo
+{
+	link_classes: Vec<usize>,
+	entry_virtual_channels: Vec<Option<usize>>,
+	cycle_per_hop: Vec<usize>,
+}
+
 ///A portion of a message. They are divided into phits.
 ///All phits must go through the same queues without phits of other packets in between.
 #[derive(Quantifiable)]
@@ -515,6 +544,8 @@ pub struct Packet
 	///The cycle when the packet has touched the first router. This is, the packet leading phit has been inserted into a router.
 	///We set it to 0 if the packet has not entered the network yet.
 	pub cycle_into_network: RefCell<usize>,
+	///Extra info tracked for some special statistics.
+	pub extra: RefCell<Option<PacketExtraInfo>>,
 }
 
 ///An application message, broken into packets
@@ -677,10 +708,23 @@ impl StatisticMeasurement
 	}
 }
 
+#[derive(Debug)]
+struct StatisticPacketMeasurement
+{
+	///The cycle in which the packet was consumed, including its tail phit.
+	consumed_cycle: usize,
+	///The number of router-to-router links the packet has traversed.
+	hops: usize,
+	///The number of cycles since the packet was created until it was consumed.
+	delay: usize,
+}
+
 ///All the global statistics captured.
 #[derive(Debug)]
 pub struct Statistics
 {
+	//The stored path is used for some calls to `config::evaluate`.
+	//path: PathBuf,
 	///The measurement since the last reset.
 	current_measurement: StatisticMeasurement,
 	///Specific statistics of the links. Indexed by router and port.
@@ -689,14 +733,32 @@ pub struct Statistics
 	temporal_step: usize,
 	///The periodic measurements requested by non-zero statistics_temporal_step.
 	temporal_statistics: Vec<StatisticMeasurement>,
+	///For each percentile `perc` write server statistics for that percentile. This is, the lowest value such that `perc`% of the servers have lower value.
+	///These values will appear in the `server_percentile{perc}` field of the result file.
+	///For example, `server_percentile25.injected_load` will be a value with 25% of the servers generating less load and `server_percentile25.accepted_load` will be a value with 25% of the servers consuming less load. Note those values will probably correspond to different servers, despite being written into the same record.
+	///The percentiles are integer numbers mostly to make obvious their representation in the name of the field.
+	///The default value is empty.
+	server_percentiles: Vec<u8>,
+	///For each percentile `perc` write packet statistics for that percentile.
+	packet_percentiles: Vec<u8>,
+	///Data collected to show `packet_percentiles` if not empty.
+	packet_statistics: Vec<StatisticPacketMeasurement>,
 	///The columns to print in the periodic reports.
 	columns: Vec<ReportColumn>,
+	///A list of statistic definitions for consumed packets.
+	///Each definition is a tuple `(keys,values)`, that are evaluated on each packet.
+	///Packets are classified via `keys` into their bin. The number of packets in each bin is counted and the associated `values` are averaged.
+	packet_defined_statistics_definitions: Vec< (Vec<Expr>,Vec<Expr>) >,
+	///For each definition of packet statistics, we have a vector with an element for each actual value of `keys`.
+	///Each of these elements have that value of `key`, together with the averages and the count.
+	packet_defined_statistics_measurement: Vec< Vec< (Vec<ConfigurationValue>,Vec<f32>,usize) >>,
 }
 
 impl Statistics
 {
-	fn new(statistics_temporal_step:usize, topology: &dyn Topology)->Statistics
+	fn new(statistics_temporal_step:usize, server_percentiles: Vec<u8>, packet_percentiles: Vec<u8>, statistics_packet_definitions:Vec<(Vec<Expr>,Vec<Expr>)>, topology: &dyn Topology)->Statistics
 	{
+		let packet_defined_statistics_measurement = vec![ vec![]; statistics_packet_definitions.len() ];
 		Statistics{
 			//begin_cycle:0,
 			//created_phits:0,
@@ -710,6 +772,9 @@ impl Statistics
 			link_statistics: (0..topology.num_routers()).map(|i| (0..topology.ports(i)).map(|_|LinkStatistics::new()).collect() ).collect(),
 			temporal_step: statistics_temporal_step,
 			temporal_statistics: vec![],
+			server_percentiles,
+			packet_percentiles,
+			packet_statistics: vec![],
 			columns: vec![
 				ReportColumnKind::BeginEndCycle.into(),
 				ReportColumnKind::InjectedLoad.into(),
@@ -721,6 +786,8 @@ impl Statistics
 				ReportColumnKind::ServerGenerationJainIndex.into(),
 				//ReportColumnKind::ServerConsumptionJainIndex.into(),
 				],
+			packet_defined_statistics_definitions:statistics_packet_definitions,
+			packet_defined_statistics_measurement,
 		}
 	}
 	///Print in stdout a header showing the statistical columns to be periodically printed.
@@ -789,7 +856,7 @@ impl Statistics
 	{
 		self.current_measurement.consumed_packets+=1;
 		let network_delay = cycle-*packet.cycle_into_network.borrow();
-		self.current_measurement.total_message_delay += network_delay;
+		self.current_measurement.total_packet_network_delay += network_delay;
 		let hops=packet.routing_info.borrow().hops;
 		self.current_measurement.total_packet_hops+=hops;
 		if self.current_measurement.total_packet_per_hop_count.len() <= hops
@@ -806,10 +873,60 @@ impl Statistics
 				self.temporal_statistics[index].begin_cycle = index*self.temporal_step;
 			}
 			self.temporal_statistics[index].consumed_packets+=1;
-			self.temporal_statistics[index].total_message_delay+=network_delay;
+			self.temporal_statistics[index].total_packet_network_delay+=network_delay;
 			self.temporal_statistics[index].total_packet_hops+=hops;
 			//Is total_packet_per_hop_count too much here?
 		}
+		if !self.packet_percentiles.is_empty()
+		{
+			self.packet_statistics.push(StatisticPacketMeasurement{consumed_cycle:cycle,hops,delay:network_delay});
+		}
+		if !self.packet_defined_statistics_definitions.is_empty()
+		{
+			let mut context_content = vec![];
+			context_content.push( (String::from("hops"), ConfigurationValue::Number(hops as f64)) );
+			context_content.push( (String::from("delay"), ConfigurationValue::Number(network_delay as f64)) );
+			context_content.push( (String::from("cycle_into_network"), ConfigurationValue::Number(*packet.cycle_into_network.borrow() as f64)) );
+			context_content.push( (String::from("size"), ConfigurationValue::Number(packet.size as f64)) );
+			let be = packet.extra.borrow();
+			let extra = be.as_ref().unwrap();
+			let link_classes = extra.link_classes.iter().map(|x|ConfigurationValue::Number(*x as f64)).collect();
+			let entry_virtual_channels = extra.entry_virtual_channels.iter().map(|x|match x{
+				Some(v) => ConfigurationValue::Number(*v as f64),
+				None => ConfigurationValue::None,
+			}).collect();
+			let cycle_per_hop = extra.cycle_per_hop.iter().map(|x|ConfigurationValue::Number(*x as f64)).collect();
+			context_content.push( (String::from("link_classes"), ConfigurationValue::Array(link_classes)) );
+			context_content.push( (String::from("entry_virtual_channels"), ConfigurationValue::Array(entry_virtual_channels)) );
+			context_content.push( (String::from("cycle_per_hop"), ConfigurationValue::Array(cycle_per_hop)) );
+			let context = ConfigurationValue::Object( String::from("packet"), context_content );
+			let path = Path::new(".");
+			for (index,definition) in self.packet_defined_statistics_definitions.iter().enumerate()
+			{
+				let key : Vec<ConfigurationValue> = definition.0.iter().map(|key_expr|config::evaluate( key_expr, &context, path)).collect();
+				let value : Vec<f32> = definition.1.iter().map(|key_expr|
+					match config::evaluate( key_expr, &context, path){
+						ConfigurationValue::Number(x) => x as f32,
+						_ => 0f32,
+					}).collect();
+				//find the measurement
+				let measurement = self.packet_defined_statistics_measurement[index].iter_mut().find(|m|m.0==key);
+				match measurement
+				{
+					Some(m) =>
+					{
+						for (iv,v) in m.1.iter_mut().enumerate()
+						{
+							*v += value[iv];
+						}
+						m.2+=1;
+					}
+					None => {
+						self.packet_defined_statistics_measurement[index].push( (key,value,1) )
+					},
+				};
+			}
+		}
 	}
 	fn track_consumed_message(&mut self, cycle:usize)
 	{
@@ -1030,6 +1147,9 @@ impl<'a> Simulation<'a>
 		let mut link_classes = None;
 		let mut statistics_temporal_step = 0;
 		let mut launch_configurations: Vec<ConfigurationValue> = vec![];
+		let mut statistics_server_percentiles: Vec<u8> = vec![];
+		let mut statistics_packet_percentiles: Vec<u8> = vec![];
+		let mut statistics_packet_definitions:Vec< (Vec<Expr>,Vec<Expr>) > = vec![];
 		if let &ConfigurationValue::Object(ref cv_name, ref cv_pairs)=cv
 		{
 			if cv_name!="Configuration"
@@ -1042,17 +1162,17 @@ impl<'a> Simulation<'a>
 				{
 					"random_seed" => match value
 					{
-						&ConfigurationValue::Number(f) => seed=Some(f as usize),
+						&ConfigurationValue::Number(f) => seed=Some(f.round() as usize),
 						_ => panic!("bad value for random_seed"),
 					}
 					"warmup" => match value
 					{
-						&ConfigurationValue::Number(f) => warmup=Some(f as usize),
+						&ConfigurationValue::Number(f) => warmup=Some(f.round() as usize),
 						_ => panic!("bad value for warmup"),
 					}
 					"measured" => match value
 					{
-						&ConfigurationValue::Number(f) => measured=Some(f as usize),
+						&ConfigurationValue::Number(f) => measured=Some(f.round() as usize),
 						_ => panic!("bad value for measured"),
 					}
 					//"topology" => topology=Some(new_topology(value)),
@@ -1064,7 +1184,7 @@ impl<'a> Simulation<'a>
 					},
 					"maximum_packet_size" => match value
 					{
-						&ConfigurationValue::Number(f) => maximum_packet_size=Some(f as usize),
+						&ConfigurationValue::Number(f) => maximum_packet_size=Some(f.round() as usize),
 						_ => panic!("bad value for maximum_packet_size"),
 					}
 					"router" => router_cfg=Some(&value),
@@ -1076,7 +1196,7 @@ impl<'a> Simulation<'a>
 					}
 					"statistics_temporal_step" => match value
 					{
-						&ConfigurationValue::Number(f) => statistics_temporal_step=f as usize,
+						&ConfigurationValue::Number(f) => statistics_temporal_step=f.round() as usize,
 						_ => panic!("bad value for statistics_temporal_step"),
 					}
 					"launch_configurations" => match value
@@ -1084,6 +1204,52 @@ impl<'a> Simulation<'a>
 						&ConfigurationValue::Array(ref l) => launch_configurations=l.clone(),
 						_ => panic!("bad value for launch_configurations"),
 					}
+					"statistics_server_percentiles" => match value
+					{
+						&ConfigurationValue::Array(ref l) => statistics_server_percentiles=l.iter().map(|v|match v{
+							&ConfigurationValue::Number(f) => u8::try_from(f.round() as u8).unwrap_or_else(|e|panic!("could not convert {} to u8 ({})",f,e)),
+							x => panic!("{} is a bad value for statistics_server_percentiles",x),
+						}).collect(),
+						_ => panic!("bad value for statistics_server_percentiles"),
+					}
+					"statistics_packet_percentiles" => match value
+					{
+						&ConfigurationValue::Array(ref l) => statistics_packet_percentiles=l.iter().map(|v|match v{
+							&ConfigurationValue::Number(f) => u8::try_from(f.round() as u8).unwrap_or_else(|e|panic!("could not convert {} to u8 ({})",f,e)),
+							x => panic!("{} is a bad value for statistics_packet_percentiles",x),
+						}).collect(),
+						_ => panic!("bad value for statistics_packet_percentiles"),
+					}
+					"statistics_packet_definitions" => match value
+					{
+						&ConfigurationValue::Array(ref l) => statistics_packet_definitions=l.iter().map(|definition|match definition {
+							&ConfigurationValue::Array(ref dl) => {
+								if dl.len()!=2
+								{
+									panic!("Each definition of statistics_packet_definitions must be composed of [keys,values]");
+								}
+								let keys = match dl[0]
+								{
+									ConfigurationValue::Array(ref lx) => lx.iter().map(|x|match x{
+										ConfigurationValue::Expression(expr) => expr.clone(),
+										_ => panic!("bad value for statistics_packet_definitions"),
+										}).collect(),
+									_ => panic!("bad value for statistics_packet_definitions"),
+								};
+								let values = match dl[1]
+								{
+									ConfigurationValue::Array(ref lx) => lx.iter().map(|x|match x{
+										ConfigurationValue::Expression(expr) => expr.clone(),
+										_ => panic!("bad value for statistics_packet_definitions"),
+										}).collect(),
+									_ => panic!("bad value for statistics_packet_definitions"),
+								};
+								(keys,values)
+							},
+							_ => panic!("bad value for statistics_packet_definitions"),
+						}).collect(),
+						_ => panic!("bad value for statistics_packet_definitions"),
+					}
 					"legend_name" => (),
 					_ => panic!("Nothing to do with field {} in Configuration",name),
 				}
@@ -1155,7 +1321,7 @@ impl<'a> Simulation<'a>
 			topology:&topology,
 			rng:&rng,
 		});
-		let statistics=Statistics::new(statistics_temporal_step,topology.as_ref());
+		let statistics=Statistics::new(statistics_temporal_step,statistics_server_percentiles,statistics_packet_percentiles,statistics_packet_definitions,topology.as_ref());
 		Simulation{
 			configuration: cv.clone(),
 			seed,
@@ -1233,6 +1399,19 @@ impl<'a> Simulation<'a>
 						&Location::RouterPort{router_index:router,router_port:port} =>
 						{
 							self.statistics.link_statistics[router][port].phit_arrivals+=1;
+							if phit.is_begin() && !self.statistics.packet_defined_statistics_definitions.is_empty()
+							{
+								let mut be = phit.packet.extra.borrow_mut();
+								if let None = *be
+								{
+									*be=Some(PacketExtraInfo::default());
+								}
+								let extra = be.as_mut().unwrap();
+								let (_,link_class) = self.network.topology.neighbour(router,port);
+								extra.link_classes.push(link_class);
+								extra.entry_virtual_channels.push(*phit.virtual_channel.borrow());
+								extra.cycle_per_hop.push(self.cycle);
+							}
 							let mut brouter=self.network.routers[router].borrow_mut();
 							brouter.insert(phit.clone(),port,&self.rng);
 							if brouter.pending_events()==0
@@ -1394,6 +1573,7 @@ impl<'a> Simulation<'a>
 							message:message.clone(),
 							index:0,
 							cycle_into_network:RefCell::new(0),
+							extra: RefCell::new(None),
 						}));
 						size-=ps;
 					}
@@ -1571,6 +1751,77 @@ impl<'a> Simulation<'a>
 			];
 			result_content.push((String::from("temporal_statistics"),ConfigurationValue::Object(String::from("TemporalStatistics"),temporal_content)));
 		}
+		if !self.statistics.server_percentiles.is_empty()
+		{
+			let mut servers_injected_load : Vec<f64> = self.network.servers.iter().map(|s|s.statistics.created_phits as f64/cycles as f64).collect();
+			let mut servers_accepted_load : Vec<f64> = self.network.servers.iter().map(|s|s.statistics.consumed_phits as f64/cycles as f64).collect();
+			let mut servers_average_message_delay : Vec<f64> = self.network.servers.iter().map(|s|s.statistics.total_message_delay as f64/s.statistics.consumed_messages as f64).collect();
+			//XXX There are more efficient ways to find percentiles than to sort them, but should not be notable in any case. See https://en.wikipedia.org/wiki/Selection_algorithm
+			servers_injected_load.sort_by(|a,b|a.partial_cmp(b).unwrap_or(Ordering::Less));
+			servers_accepted_load.sort_by(|a,b|a.partial_cmp(b).unwrap_or(Ordering::Less));
+			servers_average_message_delay.sort_by(|a,b|a.partial_cmp(b).unwrap_or(Ordering::Less));
+			for &percentile in self.statistics.server_percentiles.iter()
+			{
+				let mut index:usize = num_servers * usize::from(percentile) /100;
+				if index >= num_servers
+				{
+					//This happens at least in percentile 100%.
+					//We cannot find a value greater than ALL, just return the greatest.
+					index = num_servers -1;
+				}
+				let server_content = vec![
+					(String::from("injected_load"),ConfigurationValue::Number(servers_injected_load[index])),
+					(String::from("accepted_load"),ConfigurationValue::Number(servers_accepted_load[index])),
+					(String::from("average_message_delay"),ConfigurationValue::Number(servers_average_message_delay[index])),
+				];
+				result_content.push((format!("server_percentile{}",percentile),ConfigurationValue::Object(String::from("ServerStatistics"),server_content)));
+			}
+		}
+		if !self.statistics.packet_percentiles.is_empty()
+		{
+			let mut packets_delay : Vec<usize> = self.statistics.packet_statistics.iter().map(|ps|ps.delay).collect();
+			let mut packets_hops : Vec<usize> = self.statistics.packet_statistics.iter().map(|ps|ps.hops).collect();
+			packets_delay.sort();
+			packets_hops.sort();
+			let num_packets = packets_delay.len();
+			for &percentile in self.statistics.packet_percentiles.iter()
+			{
+				let mut index:usize = num_packets * usize::from(percentile) /100;
+				if index >= num_packets
+				{
+					//This happens at least in percentile 100%.
+					//We cannot find a value greater than ALL, just return the greatest.
+					index = num_packets -1;
+				}
+				let packet_content = vec![
+					(String::from("delay"),ConfigurationValue::Number(packets_delay[index] as f64)),
+					(String::from("hops"),ConfigurationValue::Number(packets_hops[index] as f64)),
+				];
+				result_content.push((format!("packet_percentile{}",percentile),ConfigurationValue::Object(String::from("PacketStatistics"),packet_content)));
+			}
+		}
+		if !self.statistics.packet_defined_statistics_measurement.is_empty()
+		{
+			let mut pds_content=vec![];
+			for definition_measurement in self.statistics.packet_defined_statistics_measurement.iter()
+			{
+				let mut dm_list = vec![];
+				for (key,val,count) in definition_measurement
+				{
+					let fcount = *count as f32;
+					//One average for each value field
+					let averages = ConfigurationValue::Array( val.iter().map(|v|ConfigurationValue::Number(f64::from(v/fcount))).collect() );
+					let dm_content: Vec<(String,ConfigurationValue)> = vec![
+						(String::from("key"),ConfigurationValue::Array(key.to_vec())),
+						(String::from("average"),averages),
+						(String::from("count"),ConfigurationValue::Number(*count as f64)),
+					];
+					dm_list.push( ConfigurationValue::Object(String::from("PacketBin"),dm_content) );
+				}
+				pds_content.push(ConfigurationValue::Array(dm_list));
+			}
+			result_content.push( (String::from("packet_defined_statistics"),ConfigurationValue::Array(pds_content)) );
+		}
 		let result=ConfigurationValue::Object(String::from("Result"),result_content);
 		writeln!(output,"{}",result).unwrap();
 	}
@@ -1676,23 +1927,56 @@ impl Debug for Plugs
 /// `file` must be a configuration file with the experiment to simulate.
 /// `plugs` constains the plugged builder functions.
 /// `result_file` indicates where to write the results.
-//pub fn file_main(file:&mut File, plugs:&Plugs, option_matches:&Matches)
-pub fn file_main(file:&mut File, plugs:&Plugs, mut results_file:Option<File>)
+/// `free_args` are free arguments. Those of the form `path=value` are used to override configurations.
+pub fn file_main(file:&mut File, plugs:&Plugs, mut results_file:Option<File>,free_args:&[String])
 {
 	let mut contents = String::new();
 	file.read_to_string(&mut contents).expect("something went wrong reading the file");
 
+	let mut rewrites: Vec< (Expr,ConfigurationValue) > = vec![];
+	for arg in free_args
+	{
+		if let Some( (left,right) ) = arg.split_once('=')
+		{
+			let left_expr = match config_parser::parse_expression(left).expect("error parsing a free argument")
+			{
+				config_parser::Token::Expression(expr) => expr,
+				x => panic!("the left of free argument is not an expression ({:?}), which it should be.",x),
+			};
+			//let right_expr = match config_parser::parse_expression(right).expect("error parsing a free argument")
+			//{
+			//	config_parser::Token::Expression(expr) => expr,
+			//	x => panic!("the right of free argument is not an expression ({:?}), which it should be.",x),
+			//};
+			let right_value = match config_parser::parse(right).expect("error parsing a free argument")
+			{
+				config_parser::Token::Value(value) => value,
+				x => panic!("the right of free argument is not a value ({:?}), which it should be.",x),
+			};
+			rewrites.push( (left_expr,right_value) );
+		} else {
+			println!("WARNING: ignoring argument {}",arg);
+		}
+	}
+
+	//let working_directory=std::env::current_dir().expect("Could not get working directory.");
+
 	//println!("With text:\n{}", contents);
 	match config_parser::parse(&contents)
 	{
 		Err(x) => println!("error parsing configuration file: {:?}",x),
-		Ok(x) =>
+		Ok(mut x) =>
 		{
 			println!("parsed correctly: {:?}",x);
 			match x
 			{
-				config_parser::Token::Value(ref value) =>
+				config_parser::Token::Value(ref mut value) =>
 				{
+					for (path_expr,new_value) in rewrites
+					{
+						//config::rewrite_pair(value,&path_expr,&new_value,&working_directory);
+						config::rewrite_pair_value(value,&path_expr,new_value);
+					}
 					let flat=flatten_configuration_value(value);
 					if let ConfigurationValue::Experiments(ref experiments)=flat
 					{
@@ -1728,6 +2012,11 @@ pub fn file_main(file:&mut File, plugs:&Plugs, mut results_file:Option<File>)
 //pub fn directory_main(path:&Path, binary:&str, plugs:&Plugs, option_matches:&Matches)
 pub fn directory_main(path:&Path, binary:&str, plugs:&Plugs, action:Action, options: ExperimentOptions)
 {
+	if !path.exists()
+	{
+		println!("Folder {:?} does not exists; creating it.",path);
+		fs::create_dir(&path).expect("Something went wrong when creating the main path.");
+	}
 	let binary_path=Path::new(binary);
 	//let mut experiment=Experiment::new(binary_path,path,plugs,option_matches);
 	let mut experiment=Experiment::new(binary_path,path,plugs,options);
