@@ -1,10 +1,23 @@
+/*!
+
+A policy defined to what output queue/port to request among the ones returned as possible by the routing function. Policies are designed to be applied in sequence so that there remains at most a single candidate.
+
+One should include always the policy `EnforceFlowControl` or equivalent at some point. To ensure at most one candidate you may use the `Random` policy.
+
+see [`new_virtual_channel_policy`](fn.new_virtual_channel_policy.html) for documentation on the configuration syntax of predefined policies.
+
+*/
 
 use crate::config_parser::ConfigurationValue;
 use crate::routing::CandidateEgress;
 use crate::router::Router;
 use crate::topology::{Topology,Location};
 use crate::Plugs;
+
 use std::cell::{RefCell};
+use std::fmt::Debug;
+use std::convert::TryInto;
+
 use ::rand::{Rng,StdRng};
 
 ///Extra information to be used by the policies of virtual channels.
@@ -29,9 +42,9 @@ pub struct RequestInfo<'a>
 	pub port_occupied_output_space: Option<&'a Vec<usize>>,
 	///Number of available phits in the output space of the current router at the indexed port.
 	pub port_available_output_space: Option<&'a Vec<usize>>,
-	///Number of phits currently in the output space allocated to a virtual channel. Index by [port_index][virtual_channel].
+	///Number of phits currently in the output space allocated to a virtual channel. Index by `[port_index][virtual_channel]`.
 	pub virtual_channel_occupied_output_space: Option<&'a Vec<Vec<usize>>>,
-	///Number of available phits in the output space allocated to a virtual channel. Index by [port_index][virtual_channel].
+	///Number of available phits in the output space allocated to a virtual channel. Index by `[port_index][virtual_channel]`.
 	pub virtual_channel_available_output_space: Option<&'a Vec<Vec<usize>>>,
 	///Number of cycles at the front of input space,
 	pub time_at_front: Option<usize>,
@@ -45,7 +58,7 @@ pub struct RequestInfo<'a>
 ///	rng, self.virtual_ports(credits and length), phit.packet.routing_info.borrow().hops, server_ports,
 /// topology.{distance,neighbour}, port_average_neighbour_queue_length, port_last_transmission
 ///We could also provide functions to declare which aspects must be computed. Thus allowing to both share when necessary and to not computing ti when unnecessary.
-pub trait VirtualChannelPolicy
+pub trait VirtualChannelPolicy : Debug
 {
 	///Apply the policy over a list of candidates and return the candidates that fulfil the policy requirements.
 	///candidates: the list to be filtered.
@@ -67,7 +80,192 @@ pub struct VCPolicyBuilderArgument<'a>
 	pub plugs: &'a Plugs,
 }
 
-//pub fn new_virtual_channel_policy(cv: &ConfigurationValue, plugs:&Plugs) -> Box<dyn VirtualChannelPolicy>
+/** Build a new virtual channel policy. These policies are successive refinements over the available ones as returned by the routing function, to determine to which exit a request is done.
+
+## Basic policies
+
+### Identity
+
+No operation. To be used inside meta-policies that can switch among several others.
+
+### EnforceFlowControl
+
+Filter out those candidates that have not enough credits or the equivalent depending on buffer structure. It should appear at least once in the list of policies.
+
+### Random
+
+Selects one candidate randomly among the available.
+
+### Shortest
+
+Selects the port+virtual channel with more available credits. Do not resolve ties.
+
+
+### Hops
+
+A policy to avoid deadlock. Use the virtual channel `i` in the `i`-th hop of the packet.
+
+### WideHops
+
+A variant of Hops that allow `width` VCs in each hop.
+
+```ignore
+WideHops{ width:2 }
+```
+
+## Policies measuring queues
+
+### LowestSinghWeight
+
+Select the lowest value of the product of the queue length (that is, consumed credits) times the estimated hop count (usually 1 plus the distance from next router to target router)
+This was initially proposed for the UGAL routing.
+
+```ignore
+LowestSinghWeight
+{
+	//We may add a small constant so the distance is always relevant, even for low loads
+	extra_congestion: 1,
+	//For distances is less important to add anything.
+	extra_distance: 0,
+	//Whether to aggregate all buffers in that port or to use just the space of the candidate.
+	aggregate: false,
+	///Whether to include in the computation the space of the output queue.
+	use_internal_space: true
+	///Whether to include in the computation the space in the neighbour router. It uses a credit counter as proxy.
+	use_neighbour_space: true,
+	///Whether to use the estimation of remaining hops given by the routing algorithm.
+	///Some non-minimal routing may provide that estimation, check their documentation.
+	use_estimation: true,
+}
+```
+
+### OccupancyFunction
+
+## Label manipulation
+
+Some routings can label their candidates. For example into minimal/non-minimal routes. We may use that classification to make decisions on them.
+
+### LowestLabel
+
+Select the candidate with least label (possible signifying minimal routing).
+
+### LabelSaturate
+
+Apply the transformation `New label = min{old_label,value} or max{old_label,value}`
+
+```ignore
+LabelSaturate
+{
+	value: 1,
+	bottom: true,
+}
+```
+
+### LabelTransform
+
+More advanced transformations. Lineal operation with optional saturation (reducing/raising the value) and limits (filtering out).
+
+```ignore
+LabelTransform
+{
+	multiplier: 1,
+	summand: 1,
+	saturate_bottom: 0,
+	//saturate_top
+	//minimum
+	maximum: 2,
+}
+```
+
+### NegateLabel
+
+Negate the label. Alternatively use `LabelTransform` with `multiplier:-1`.
+
+### VecLabel
+
+Apply a map to the label, i.e., `new_label = vector[old_label]`.
+
+```ignore
+VecLabel
+{
+	label_vector: [1, 0],
+}
+```
+
+### MapLabel
+
+A meta-policy applying a different policy to candidates with each label.
+
+```ignore
+MapLabel
+{
+	//Apply Identity to label 0.
+	//Apply Random to label 1.
+	label_to_policy: [Identity, Random],
+	//We may apply a policy to negative labels
+	//below_policy: Identity,
+	//We may apply a policy to label values over the range
+	//above_policy: Identity,
+}
+```
+
+## Purely VC transformations
+
+### ShiftEntryVC
+
+Only allows those candidates whose vc equals their entry vc plus some `s` in `shifts`. This is very similar to the `Hops` policy, but can be combined with other policies. For example, to increase VC only in a escape sub-network.
+
+```ignore
+ShiftEntryVC
+{
+	shifts: [1],
+}
+```
+
+### ArgumentVC
+
+Only allows those candidates whose vc is in the allowed list. To be used inside meta-policies.
+
+```ignore
+AgumentVC
+{
+	allowed: [0, 1]
+}
+```
+
+### MapEntryVC
+
+A meta-policy applying a different policy to candidates from each entry virtual channel.
+
+```ignore
+MapEntryVC
+{
+	//Do nothing over the two first VCs
+	vc_to_policy: [Identity, Identity]
+	//In the other VCs the packet must increase each hop.
+	above_policy: ShiftEntryVC{shifts:[1]},
+}
+```
+
+## Hop based
+
+Policies that use the number of hops given by the packet. We have already commented on `Hops` and `WideHops`.
+
+### MapHop
+
+Meta-policy applying a different policy to each hop.
+
+```ignore
+MapHop
+{
+	//First hop from a router to another must be in VC 0 or 1.
+	hop_to_policy: [ArgumentVC{allowed:[0,1]}],
+	//Further hops increase the VC number by 1.
+	above_policy: ShiftEntryVC{shifts:[1]},
+}
+```
+
+*/
 pub fn new_virtual_channel_policy(arg:VCPolicyBuilderArgument) -> Box<dyn VirtualChannelPolicy>
 {
 	if let &ConfigurationValue::Object(ref cv_name, ref _cv_pairs)=arg.cv
@@ -79,6 +277,7 @@ pub fn new_virtual_channel_policy(arg:VCPolicyBuilderArgument) -> Box<dyn Virtua
 		};
 		match cv_name.as_ref()
 		{
+			"Identity" => Box::new(Identity::new(arg)),
 			"Random" => Box::new(Random::new(arg)),
 			"Shortest" => Box::new(Shortest::new(arg)),
 			"Hops" => Box::new(Hops::new(arg)),
@@ -91,15 +290,57 @@ pub fn new_virtual_channel_policy(arg:VCPolicyBuilderArgument) -> Box<dyn Virtua
 			"OccupancyFunction" => Box::new(OccupancyFunction::new(arg)),
 			"NegateLabel" => Box::new(NegateLabel::new(arg)),
 			"VecLabel" => Box::new(VecLabel::new(arg)),
+			"MapLabel" => Box::new(MapLabel::new(arg)),
+			"ShiftEntryVC" => Box::new(ShiftEntryVC::new(arg)),
+			"MapHop" => Box::new(MapHop::new(arg)),
+			"ArgumentVC" => Box::new(ArgumentVC::new(arg)),
+			"Either" => Box::new(Either::new(arg)),
+			"MapEntryVC" => Box::new(MapEntryVC::new(arg)),
 			_ => panic!("Unknown traffic {}",cv_name),
 		}
 	}
 	else
 	{
-		panic!("Trying to create a traffic from a non-Object");
+		panic!("Trying to create a policy from a non-Object\narg={:?}",arg);
 	}
 }
 
+///Does not do anything. Just a placeholder for some operations.
+#[derive(Debug)]
+pub struct Identity{}
+
+impl VirtualChannelPolicy for Identity
+{
+	fn filter(&self, candidates:Vec<CandidateEgress>, _router:&dyn Router, _info: &RequestInfo, _topology:&dyn Topology, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	{
+		candidates
+	}
+
+	fn need_server_ports(&self)->bool
+	{
+		false
+	}
+
+	fn need_port_average_queue_length(&self)->bool
+	{
+		false
+	}
+
+	fn need_port_last_transmission(&self)->bool
+	{
+		false
+	}
+}
+
+impl Identity
+{
+	pub fn new(_arg:VCPolicyBuilderArgument) -> Identity
+	{
+		Identity{}
+	}
+}
+
+
 ///Request a port+virtual channel at random from all available.
 #[derive(Debug)]
 pub struct Random{}
@@ -126,7 +367,6 @@ impl VirtualChannelPolicy for Random
 	{
 		false
 	}
-
 }
 
 impl Random
@@ -1197,7 +1437,7 @@ impl NegateLabel
 
 
 ///Vector of labels
-///new_label = vector[old_label]
+///`new_label = vector[old_label]`
 #[derive(Debug)]
 pub struct VecLabel
 {
@@ -1286,3 +1526,550 @@ impl VecLabel
 	}
 }
 
+///Apply a different policy to candidates with each label.
+#[derive(Debug)]
+pub struct MapLabel
+{
+	label_to_policy: Vec<Box<dyn VirtualChannelPolicy>>,
+	below_policy: Box<dyn VirtualChannelPolicy>,
+	above_policy: Box<dyn VirtualChannelPolicy>,
+}
+
+impl VirtualChannelPolicy for MapLabel
+{
+	fn filter(&self, candidates:Vec<CandidateEgress>, router:&dyn Router, info: &RequestInfo, topology:&dyn Topology, rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	{
+		//let port_average_neighbour_queue_length=port_average_neighbour_queue_length.as_ref().expect("port_average_neighbour_queue_length have not been computed for policy MapLabel");
+		let dist=topology.distance(router.get_index().expect("we need routers with index"),info.target_router_index);
+		if dist==0
+		{
+			//do nothing
+			candidates
+		}
+		else
+		{
+			let n = self.label_to_policy.len();
+			//above goes into candidate_map[ labels  ]
+			//below goes into candidate_map[ labels+1  ]
+			let mut candidate_map = vec![vec![];n+2];
+			for cand in candidates.into_iter()
+			{
+				let label : usize = if cand.label < 0
+				{
+					self.label_to_policy.len()+1
+				} else if cand.label > n.try_into().unwrap() {
+					n
+				} else {
+					cand.label.try_into().unwrap()
+				};
+				candidate_map[label].push(cand);
+			}
+			let mut policies = self.label_to_policy.iter().chain( vec![&self.above_policy].into_iter() ).chain( vec![&self.below_policy].into_iter() );
+			let mut r = vec![];
+			for candidate_list in candidate_map
+			{
+				let policy : &dyn VirtualChannelPolicy = policies.next().unwrap().as_ref();
+				r.extend( policy.filter(candidate_list,router,info,topology,rng)  );
+			}
+			r
+		}
+	}
+
+	fn need_server_ports(&self)->bool
+	{
+		true
+	}
+
+	fn need_port_average_queue_length(&self)->bool
+	{
+		true
+	}
+
+	fn need_port_last_transmission(&self)->bool
+	{
+		true
+	}
+
+}
+
+impl MapLabel
+{
+	pub fn new(arg:VCPolicyBuilderArgument) -> MapLabel
+	{
+		let mut label_to_policy=None;
+		let mut below_policy : Box<dyn VirtualChannelPolicy> =Box::new(Identity{});
+		let mut above_policy : Box<dyn VirtualChannelPolicy> =Box::new(Identity{});
+		if let &ConfigurationValue::Object(ref cv_name, ref cv_pairs)=arg.cv
+		{
+			if cv_name!="MapLabel"
+			{
+				panic!("A MapLabel must be created from a `MapLabel` object not `{}`",cv_name);
+			}
+			for &(ref name,ref value) in cv_pairs
+			{
+				match AsRef::<str>::as_ref(&name)
+				{
+ 					"label_to_policy" => match value
+ 					{
+						&ConfigurationValue::Array(ref l) => label_to_policy=Some(l.iter().map(|v| match v{
+							&ConfigurationValue::Object(_,_) => new_virtual_channel_policy(VCPolicyBuilderArgument{cv:v,..arg}),
+							_ => panic!("bad value for label_to_policy"),
+						}).collect()),
+ 						_ => panic!("bad value for label_to_policy"),
+ 					}
+					"below_policy" => match value
+					{
+						&ConfigurationValue::Object(_,_) => below_policy = new_virtual_channel_policy(VCPolicyBuilderArgument{cv:value,..arg}),
+ 						_ => panic!("bad value for below_policy"),
+					}
+					"above_policy" => match value
+					{
+						&ConfigurationValue::Object(_,_) => above_policy = new_virtual_channel_policy(VCPolicyBuilderArgument{cv:value,..arg}),
+ 						_ => panic!("bad value for above_policy"),
+					}
+					_ => panic!("Nothing to do with field {} in MapLabel",name),
+				}
+			}
+		}
+		else
+		{
+			panic!("Trying to create a MapLabel from a non-Object");
+		}
+		let label_to_policy=label_to_policy.expect("There were no label_to_policy");
+		MapLabel{
+			label_to_policy,
+			below_policy,
+			above_policy,
+		}
+	}
+}
+
+
+///Only allows those candidates whose vc equals their entry vc plus some `s` in `shifts`.
+#[derive(Debug)]
+pub struct ShiftEntryVC
+{
+	shifts: Vec<i32>,
+}
+
+impl VirtualChannelPolicy for ShiftEntryVC
+{
+	fn filter(&self, candidates:Vec<CandidateEgress>, router:&dyn Router, info: &RequestInfo, topology:&dyn Topology, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	{
+		//let port_average_neighbour_queue_length=port_average_neighbour_queue_length.as_ref().expect("port_average_neighbour_queue_length have not been computed for policy ShiftEntryVC");
+		let dist=topology.distance(router.get_index().expect("we need routers with index"),info.target_router_index);
+		if dist==0
+		{
+			//do nothing
+			candidates
+		}
+		else
+		{
+			let evc = info.entry_virtual_channel as i32;
+			candidates.into_iter().filter(|&CandidateEgress{virtual_channel,..}| self.shifts.contains(&(virtual_channel as i32-evc)) ).collect::<Vec<_>>()
+		}
+	}
+
+	fn need_server_ports(&self)->bool
+	{
+		false
+	}
+
+	fn need_port_average_queue_length(&self)->bool
+	{
+		false
+	}
+
+	fn need_port_last_transmission(&self)->bool
+	{
+		false
+	}
+
+}
+
+impl ShiftEntryVC
+{
+	pub fn new(arg:VCPolicyBuilderArgument) -> ShiftEntryVC
+	{
+		let mut shifts=None;
+		if let &ConfigurationValue::Object(ref cv_name, ref cv_pairs)=arg.cv
+		{
+			if cv_name!="ShiftEntryVC"
+			{
+				panic!("A ShiftEntryVC must be created from a `ShiftEntryVC` object not `{}`",cv_name);
+			}
+			for &(ref name,ref value) in cv_pairs
+			{
+				match AsRef::<str>::as_ref(&name)
+				{
+ 					"shifts" => match value
+ 					{
+						&ConfigurationValue::Array(ref l) => shifts=Some(l.iter().map(|v| match v{
+							&ConfigurationValue::Number(x) => x as i32,
+							_ => panic!("bad value for shifts"),
+						}).collect()),
+ 						_ => panic!("bad value for shifts"),
+ 					}
+					_ => panic!("Nothing to do with field {} in ShiftEntryVC",name),
+				}
+			}
+		}
+		else
+		{
+			panic!("Trying to create a ShiftEntryVC from a non-Object");
+		}
+		let shifts=shifts.expect("There were no shifts");
+		ShiftEntryVC{
+			shifts,
+		}
+	}
+}
+
+
+///Apply a different policy to each hop.
+#[derive(Debug)]
+pub struct MapHop
+{
+	hop_to_policy: Vec<Box<dyn VirtualChannelPolicy>>,
+	above_policy: Box<dyn VirtualChannelPolicy>,
+}
+
+impl VirtualChannelPolicy for MapHop
+{
+	fn filter(&self, candidates:Vec<CandidateEgress>, router:&dyn Router, info: &RequestInfo, topology:&dyn Topology, rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	{
+		//let port_average_neighbour_queue_length=port_average_neighbour_queue_length.as_ref().expect("port_average_neighbour_queue_length have not been computed for policy MapHop");
+		let dist=topology.distance(router.get_index().expect("we need routers with index"),info.target_router_index);
+		if dist==0
+		{
+			//do nothing
+			candidates
+		}
+		else
+		{
+			let policy = if info.performed_hops>=self.hop_to_policy.len() { &self.above_policy } else { &self.hop_to_policy[info.performed_hops] };
+			policy.filter(candidates,router,info,topology,rng)
+		}
+	}
+
+	fn need_server_ports(&self)->bool
+	{
+		true
+	}
+
+	fn need_port_average_queue_length(&self)->bool
+	{
+		true
+	}
+
+	fn need_port_last_transmission(&self)->bool
+	{
+		true
+	}
+
+}
+
+impl MapHop
+{
+	pub fn new(arg:VCPolicyBuilderArgument) -> MapHop
+	{
+		let mut hop_to_policy=None;
+		let mut above_policy : Box<dyn VirtualChannelPolicy> =Box::new(Identity{});
+		if let &ConfigurationValue::Object(ref cv_name, ref cv_pairs)=arg.cv
+		{
+			if cv_name!="MapHop"
+			{
+				panic!("A MapHop must be created from a `MapHop` object not `{}`",cv_name);
+			}
+			for &(ref name,ref value) in cv_pairs
+			{
+				match AsRef::<str>::as_ref(&name)
+				{
+ 					"hop_to_policy" => match value
+ 					{
+						&ConfigurationValue::Array(ref l) => hop_to_policy=Some(l.iter().map(|v| match v{
+							&ConfigurationValue::Object(_,_) => new_virtual_channel_policy(VCPolicyBuilderArgument{cv:v,..arg}),
+							_ => panic!("bad value for hop_to_policy"),
+						}).collect()),
+ 						_ => panic!("bad value for hop_to_policy"),
+ 					}
+					"above_policy" => match value
+					{
+						&ConfigurationValue::Object(_,_) => above_policy = new_virtual_channel_policy(VCPolicyBuilderArgument{cv:value,..arg}),
+ 						_ => panic!("bad value for above_policy"),
+					}
+					_ => panic!("Nothing to do with field {} in MapHop",name),
+				}
+			}
+		}
+		else
+		{
+			panic!("Trying to create a MapHop from a non-Object");
+		}
+		let hop_to_policy=hop_to_policy.expect("There were no hop_to_policy");
+		MapHop{
+			hop_to_policy,
+			above_policy,
+		}
+	}
+}
+
+///Only allows those candidates whose vc is in the allowed list.
+#[derive(Debug)]
+pub struct ArgumentVC
+{
+	allowed: Vec<usize>,
+}
+
+impl VirtualChannelPolicy for ArgumentVC
+{
+	fn filter(&self, candidates:Vec<CandidateEgress>, router:&dyn Router, info: &RequestInfo, _topology:&dyn Topology, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	{
+		//let port_average_neighbour_queue_length=port_average_neighbour_queue_length.as_ref().expect("port_average_neighbour_queue_length have not been computed for policy ArgumentVC");
+		if router.get_index().expect("we need routers with index") == info.target_router_index
+		{
+			//do nothing
+			candidates
+		}
+		else
+		{
+			candidates.into_iter().filter(|&CandidateEgress{virtual_channel,..}| self.allowed.contains(&virtual_channel) ).collect::<Vec<_>>()
+		}
+	}
+
+	fn need_server_ports(&self)->bool
+	{
+		false
+	}
+
+	fn need_port_average_queue_length(&self)->bool
+	{
+		false
+	}
+
+	fn need_port_last_transmission(&self)->bool
+	{
+		false
+	}
+
+}
+
+impl ArgumentVC
+{
+	pub fn new(arg:VCPolicyBuilderArgument) -> ArgumentVC
+	{
+		let mut allowed=None;
+		if let &ConfigurationValue::Object(ref cv_name, ref cv_pairs)=arg.cv
+		{
+			if cv_name!="ArgumentVC"
+			{
+				panic!("A ArgumentVC must be created from a `ArgumentVC` object not `{}`",cv_name);
+			}
+			for &(ref name,ref value) in cv_pairs
+			{
+				match AsRef::<str>::as_ref(&name)
+				{
+ 					"allowed" => match value
+ 					{
+						&ConfigurationValue::Array(ref l) => allowed=Some(l.iter().map(|v| match v{
+							&ConfigurationValue::Number(x) => x as usize,
+							_ => panic!("bad value for allowed"),
+						}).collect()),
+ 						_ => panic!("bad value for allowed"),
+ 					}
+					_ => panic!("Nothing to do with field {} in ArgumentVC",name),
+				}
+			}
+		}
+		else
+		{
+			panic!("Trying to create a ArgumentVC from a non-Object");
+		}
+		let allowed=allowed.expect("There were no allowed");
+		ArgumentVC{
+			allowed,
+		}
+	}
+}
+
+///Apply a different policy to each hop.
+#[derive(Debug)]
+pub struct Either
+{
+	policies: Vec<Box<dyn VirtualChannelPolicy>>,
+}
+
+impl VirtualChannelPolicy for Either
+{
+	fn filter(&self, candidates:Vec<CandidateEgress>, router:&dyn Router, info: &RequestInfo, topology:&dyn Topology, rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	{
+		//let port_average_neighbour_queue_length=port_average_neighbour_queue_length.as_ref().expect("port_average_neighbour_queue_length have not been computed for policy Either");
+		if router.get_index().expect("we need routers with index") == info.target_router_index
+		{
+			//do nothing
+			candidates
+		}
+		else
+		{
+			let mut r = Vec::new();
+			for policy in self.policies.iter()
+			{
+				r.extend(policy.as_ref().filter(candidates.clone(),router,info,topology,rng));
+			}
+			r
+		}
+	}
+
+	fn need_server_ports(&self)->bool
+	{
+		true
+	}
+
+	fn need_port_average_queue_length(&self)->bool
+	{
+		true
+	}
+
+	fn need_port_last_transmission(&self)->bool
+	{
+		true
+	}
+
+}
+
+impl Either
+{
+	pub fn new(arg:VCPolicyBuilderArgument) -> Either
+	{
+		let mut policies=None;
+		if let &ConfigurationValue::Object(ref cv_name, ref cv_pairs)=arg.cv
+		{
+			if cv_name!="Either"
+			{
+				panic!("A Either must be created from a `Either` object not `{}`",cv_name);
+			}
+			for &(ref name,ref value) in cv_pairs
+			{
+				match AsRef::<str>::as_ref(&name)
+				{
+ 					"policies" => match value
+ 					{
+						&ConfigurationValue::Array(ref l) => policies=Some(l.iter().map(|v| match v{
+							&ConfigurationValue::Object(_,_) => new_virtual_channel_policy(VCPolicyBuilderArgument{cv:v,..arg}),
+							_ => panic!("bad value for policies"),
+						}).collect()),
+ 						_ => panic!("bad value for policies"),
+ 					}
+					_ => panic!("Nothing to do with field {} in Either",name),
+				}
+			}
+		}
+		else
+		{
+			panic!("Trying to create a Either from a non-Object");
+		}
+		let policies=policies.expect("There were no policies");
+		Either{
+			policies,
+		}
+	}
+}
+
+///Apply a different policy to candidates from each entry virtual channel.
+#[derive(Debug)]
+pub struct MapEntryVC
+{
+	///Which policy to apply, index by the entry virtual channel.
+	vc_to_policy: Vec<Box<dyn VirtualChannelPolicy>>,
+	///Policy to apply if entry virtual channel is above the array range limit.
+	above_policy: Box<dyn VirtualChannelPolicy>,
+}
+
+impl VirtualChannelPolicy for MapEntryVC
+{
+	fn filter(&self, candidates:Vec<CandidateEgress>, router:&dyn Router, info: &RequestInfo, topology:&dyn Topology, rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	{
+		//let port_average_neighbour_queue_length=port_average_neighbour_queue_length.as_ref().expect("port_average_neighbour_queue_length have not been computed for policy MapEntryVC");
+		let dist=topology.distance(router.get_index().expect("we need routers with index"),info.target_router_index);
+		if dist==0
+		{
+			//do nothing
+			candidates
+		}
+		else
+		{
+			let n = self.vc_to_policy.len();
+			let evc = info.entry_virtual_channel;
+			let policy = if evc<n
+			{
+				&self.vc_to_policy[evc]
+			} else {
+				&self.above_policy
+			};
+			policy.filter(candidates,router,info,topology,rng)
+		}
+	}
+
+	fn need_server_ports(&self)->bool
+	{
+		true
+	}
+
+	fn need_port_average_queue_length(&self)->bool
+	{
+		true
+	}
+
+	fn need_port_last_transmission(&self)->bool
+	{
+		true
+	}
+
+}
+
+impl MapEntryVC
+{
+	pub fn new(arg:VCPolicyBuilderArgument) -> MapEntryVC
+	{
+		let mut vc_to_policy=None;
+		let mut above_policy : Box<dyn VirtualChannelPolicy> =Box::new(Identity{});
+		if let &ConfigurationValue::Object(ref cv_name, ref cv_pairs)=arg.cv
+		{
+			if cv_name!="MapEntryVC"
+			{
+				panic!("A MapEntryVC must be created from a `MapEntryVC` object not `{}`",cv_name);
+			}
+			for &(ref name,ref value) in cv_pairs
+			{
+				match AsRef::<str>::as_ref(&name)
+				{
+ 					"vc_to_policy" => match value
+ 					{
+						&ConfigurationValue::Array(ref l) => vc_to_policy=Some(l.iter().map(|v| match v{
+							&ConfigurationValue::Object(_,_) => new_virtual_channel_policy(VCPolicyBuilderArgument{cv:v,..arg}),
+							_ => panic!("bad value for vc_to_policy"),
+						}).collect()),
+ 						_ => panic!("bad value for vc_to_policy"),
+ 					}
+					"above_policy" => match value
+					{
+						&ConfigurationValue::Object(_,_) => above_policy = new_virtual_channel_policy(VCPolicyBuilderArgument{cv:value,..arg}),
+ 						_ => panic!("bad value for above_policy"),
+					}
+					_ => panic!("Nothing to do with field {} in MapEntryVC",name),
+				}
+			}
+		}
+		else
+		{
+			panic!("Trying to create a MapEntryVC from a non-Object");
+		}
+		let vc_to_policy=vc_to_policy.expect("There were no vc_to_policy");
+		MapEntryVC{
+			vc_to_policy,
+			above_policy,
+		}
+	}
+}
+
+
