@@ -587,7 +587,20 @@ impl<TM:'static+TransmissionMechanism> Eventful for Basic<TM>
 			Some(self.transmission_port_status.iter().map(|ref p|{
 				//let total=p.iter().map(|ref vp|self.buffer_size - vp.neighbour_credits).sum::<usize>();
 				//(total as f32) / (p.len() as f32)
-				let total=(0..amount_virtual_channels).map(|vc|self.buffer_size-p.known_available_space_for_virtual_channel(vc).expect("needs to know available space")).sum::<usize>();
+				let total=(0..amount_virtual_channels).map(|vc|{
+					//self.buffer_size-p.known_available_space_for_virtual_channel(vc).expect("needs to know available space")
+					let available = p.known_available_space_for_virtual_channel(vc).expect("needs to know available space");
+					if available>self.buffer_size
+					{
+						//panic!("We should never have more available space than the buffer size.");
+						//Actually when the neighbour is a server it may have longer queue.
+						0
+					}
+					else
+					{
+						self.buffer_size - available
+					}
+				}).sum::<usize>();
 				(total as f32) / (amount_virtual_channels as f32)
 			}).collect())
 		}
@@ -667,7 +680,18 @@ impl<TM:'static+TransmissionMechanism> Eventful for Basic<TM>
 							Location::RouterPort{router_index,router_port:_} =>router_index,
 							_ => panic!("The server is not attached to a router"),
 						};
-						let mut good_ports=simulation.routing.next(phit.packet.routing_info.borrow().deref(),simulation.network.topology.as_ref(),self.router_index,target_server,amount_virtual_channels,&simulation.rng).into_iter().filter_map(|candidate|{
+						let routing_candidates=simulation.routing.next(phit.packet.routing_info.borrow().deref(),simulation.network.topology.as_ref(),self.router_index,target_server,amount_virtual_channels,&simulation.rng);
+						let routing_idempotent = routing_candidates.idempotent;
+						if routing_candidates.len()==0
+						{
+							if routing_idempotent
+							{
+								panic!("There are no choices for packet {:?} entry_port={} entry_vc={} in router {} towards server {}",phit.packet,entry_port,entry_vc,self.router_index,target_server);
+							}
+							//There are currently no good port choices, but there may be in the future.
+							continue;
+						}
+						let mut good_ports=routing_candidates.into_iter().filter_map(|candidate|{
 							let CandidateEgress{port:f_port,virtual_channel:f_virtual_channel,..} = candidate;
 							//We analyze each candidate output port, considering whether they are in use (port or virtual channel).
 							match self.selected_input[f_port][f_virtual_channel]
@@ -697,11 +721,6 @@ impl<TM:'static+TransmissionMechanism> Eventful for Basic<TM>
 								}
 							}
 						}).collect::<Vec<_>>();
-						if good_ports.len()==0
-						{
-							//There are currently no good port choices, but there may be in the future.
-							continue;
-						}
 						let performed_hops=phit.packet.routing_info.borrow().hops;
 						//Apply all the declared virtual channel policies in order.
 						let request_info=RequestInfo{
