@@ -4,7 +4,7 @@ use ::rand::{Rng,StdRng};
 use quantifiable_derive::Quantifiable;//the derive macro
 use crate::config_parser::ConfigurationValue;
 use crate::topology::{Topology,Location};
-use crate::routing::{RoutingInfo,Routing,CandidateEgress,RoutingBuilderArgument};
+use crate::routing::{RoutingInfo,Routing,CandidateEgress,RoutingBuilderArgument,RoutingNextCandidates};
 
 ///A Cartesian ortahedral region of arbitrary dimension.
 #[derive(Quantifiable)]
@@ -137,9 +137,12 @@ impl Topology for Mesh
 	{
 		self.cartesian_data.sides.iter().map(|s|s-1).sum()
 	}
-	fn distance(&self,_origin:usize,_destination:usize) -> usize
+	fn distance(&self,origin:usize,destination:usize) -> usize
 	{
-		unimplemented!();
+		let coord_origin=self.cartesian_data.unpack(origin);
+		let coord_destination=self.cartesian_data.unpack(destination);
+		let rr=self.coordinated_routing_record(&coord_origin,&coord_destination,None);
+		rr.iter().map(|x|x.abs() as usize).sum()
 	}
 	fn amount_shortest_paths(&self,_origin:usize,_destination:usize) -> usize
 	{
@@ -650,7 +653,7 @@ pub struct DOR
 impl Routing for DOR
 {
 	//type info=CartesianRoutingRecord;
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		//let routing_record=&routing_info.routing_record.expect("DOR requires a routing record");
 		let routing_record=if let Some(ref rr)=routing_info.routing_record
@@ -679,7 +682,8 @@ impl Routing for DOR
 					{
 						//return vec![i];
 						//return (0..num_virtual_channels).map(|vc|(i,vc)).collect();
-						return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						let r= (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{candidates:r,idempotent:true};
 					}
 				}
 			}
@@ -752,7 +756,8 @@ impl Routing for DOR
 				panic!("No links improving {} dimension\n",i);
 			}
 			//return (0..num_virtual_channels).flat_map(|vc| best.iter().map(|p|(*p,vc)).collect::<Vec<(usize,usize)>>()).collect();
-			return (0..num_virtual_channels).flat_map(|vc| best.iter().map(|p|CandidateEgress::new(*p,vc)).collect::<Vec<_>>()).collect();
+			let r= (0..num_virtual_channels).flat_map(|vc| best.iter().map(|p|CandidateEgress::new(*p,vc)).collect::<Vec<_>>()).collect();
+			return RoutingNextCandidates{candidates:r,idempotent:true};
 		}
 	}
 	//fn initialize_routing_info(&self, routing_info:&mut RoutingInfo, toology:&dyn Topology, current_router:usize, target_server:usize)
@@ -878,8 +883,8 @@ impl DOR
 }
 
 /// Valiant DOR. Proposed by Valiant for Multidimensional grids. Generally you should randomize n-1 dimensions, thereby employing shortest routes when the topology is just a path.
-/// routing_info.selections=Some([k,r]) indicates that the `next` call should go toward r at dimension randomized[k]. r having been selected randomly previously.
-/// routing_info.selections=None indicates to behave as DOR.
+/// `routing_info.selections=Some([k,r])` indicates that the `next` call should go toward `r` at dimension `randomized[k]`. `r` having been selected randomly previously.
+/// `routing_info.selections=None` indicates to behave as DOR.
 #[derive(Debug)]
 pub struct ValiantDOR
 {
@@ -897,7 +902,7 @@ pub struct ValiantDOR
 
 impl Routing for ValiantDOR
 {
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		//let routing_record=&routing_info.routing_record.expect("ValiantDOR requires a routing record");
 		let mut random_amount=0i32;
@@ -938,7 +943,8 @@ impl Routing for ValiantDOR
 					{
 						//return vec![i];
 						//return (0..num_virtual_channels).map(|vc|(i,vc)).collect();
-						return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						let r= (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{candidates:r,idempotent:true};
 					}
 				}
 			}
@@ -1040,13 +1046,15 @@ impl Routing for ValiantDOR
 				//XXX not worth to box the closure, right?
 				let vcs=(0..num_virtual_channels).filter(|vc|!self.shortest_reserved_virtual_channels.contains(vc));
 				//return vcs.flat_map(|vc| best.iter().map(|p|(*p,vc)).collect::<Vec<(usize,usize)>>()).collect();
-				return vcs.flat_map(|vc| best.iter().map(|p|CandidateEgress::new(*p,vc)).collect::<Vec<_>>()).collect();
+				let r= vcs.flat_map(|vc| best.iter().map(|p|CandidateEgress::new(*p,vc)).collect::<Vec<_>>()).collect();
+				return RoutingNextCandidates{candidates:r,idempotent:true}
 			}
 			else
 			{
 				let vcs=(0..num_virtual_channels).filter(|vc|!self.randomized_reserved_virtual_channels.contains(vc));
 				//return vcs.flat_map(|vc| best.iter().map(|p|(*p,vc)).collect::<Vec<(usize,usize)>>()).collect();
-				return vcs.flat_map(|vc| best.iter().map(|p|CandidateEgress::new(*p,vc)).collect::<Vec<_>>()).collect();
+				let r= vcs.flat_map(|vc| best.iter().map(|p|CandidateEgress::new(*p,vc)).collect::<Vec<_>>()).collect();
+				return RoutingNextCandidates{candidates:r,idempotent:true}
 			};
 		}
 	}
@@ -1283,22 +1291,22 @@ impl ValiantDOR
 }
 
 
-///The O1TTURN routing uses DOR order [0,1] for some virtual channels and order [1,0] for others.
-///By default it reserves the channel 0 for [0,1] and the channel 1 for [1,0].
+///The O1TTURN routing uses DOR order `[0,1]` for some virtual channels and order `[1,0]` for others.
+///By default it reserves the channel 0 for `[0,1]` and the channel 1 for `[1,0]`.
 #[derive(Debug)]
 pub struct O1TURN
 {
 	/// Virtual channels reserved exclusively for the 0 before 1 DOR selection.
-	/// Defaults to [0]
+	/// Defaults to `[0]`
 	reserved_virtual_channels_order01: Vec<usize>,
 	/// Virtual channels reserved exclusively for the 1 before 0 DOR selection.
-	/// Defaults to [1]
+	/// Defaults to `[1]`
 	reserved_virtual_channels_order10: Vec<usize>,
 }
 
 impl Routing for O1TURN
 {
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		//let routing_record=&routing_info.routing_record.expect("DOR requires a routing record");
 		let routing_record=if let Some(ref rr)=routing_info.routing_record
@@ -1343,7 +1351,8 @@ impl Routing for O1TURN
 					if server==target_server
 					{
 						//return vec![CandidateEgress::new(i,s)];
-						return available_virtual_channels.map(|vc| CandidateEgress::new(i,vc)).collect();
+						let r= available_virtual_channels.map(|vc| CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{candidates:r,idempotent:true};
 					}
 				}
 			}
@@ -1363,7 +1372,8 @@ impl Routing for O1TURN
 				2*i+1
 			};
 			//return vec![CandidateEgress::new(p,s)];
-			return available_virtual_channels.map(|vc| CandidateEgress::new(p,vc)).collect();
+			let r= available_virtual_channels.map(|vc| CandidateEgress::new(p,vc)).collect();
+			return RoutingNextCandidates{candidates:r,idempotent:true};
 		}
 	}
 	//fn initialize_routing_info(&self, routing_info:&mut RoutingInfo, toology:&dyn Topology, current_router:usize, target_server:usize)
@@ -1480,7 +1490,7 @@ impl O1TURN
 
 
 /// Routing part of the Omni-dimensional Weighted Adaptive Routing of Nic McDonald et al.
-/// Stores RoutingInfo.selections=Some(vec![available_deroutes]).
+/// Stores `RoutingInfo.selections=Some(vec![available_deroutes])`.
 /// Only paths of currently unaligned dimensions are valid. Otherwise dimensions are ignored.
 #[derive(Debug)]
 pub struct OmniDimensionalDeroute
@@ -1493,7 +1503,7 @@ pub struct OmniDimensionalDeroute
 
 impl Routing for OmniDimensionalDeroute
 {
-	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> Vec<CandidateEgress>
+	fn next(&self, routing_info:&RoutingInfo, topology:&dyn Topology, current_router:usize, target_server:usize, num_virtual_channels:usize, _rng: &RefCell<StdRng>) -> RoutingNextCandidates
 	{
 		let (target_location,_link_class)=topology.server_neighbour(target_server);
 		let target_router=match target_location
@@ -1512,7 +1522,8 @@ impl Routing for OmniDimensionalDeroute
 					if server==target_server
 					{
 						//return (0..num_virtual_channels).map(|vc|(i,vc)).collect();
-						return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						//return (0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect();
+						return RoutingNextCandidates{candidates:(0..num_virtual_channels).map(|vc|CandidateEgress::new(i,vc)).collect(),idempotent:true}
 					}
 				}
 			}
@@ -1573,7 +1584,7 @@ impl Routing for OmniDimensionalDeroute
 				}
 			}
 		}
-		r
+		RoutingNextCandidates{candidates:r,idempotent:true}
 	}
 	//fn initialize_routing_info(&self, routing_info:&mut RoutingInfo, toology:&dyn Topology, current_router:usize, target_server:usize)
 	fn initialize_routing_info(&self, routing_info:&RefCell<RoutingInfo>, _topology:&dyn Topology, _current_router:usize, _target_server:usize, _rng: &RefCell<StdRng>)
